<?php

namespace Drupal\dofacshs\Form;

use Drupal\Core\Entity\ContentEntityType;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\field\Entity\FieldConfig;

/**
 * Configure example settings for this site.
 */
class Settings extends ConfigFormBase {

  /** 
   * Config settings.
   *
   * @var string
   */
  const SETTINGS = 'dofacshs.settings';

  /** 
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'dofacshs_settings';
  }

  /** 
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      static::SETTINGS,
    ];
  }

  /** 
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config(static::SETTINGS);
    /** @var \Drupal\Core\Entity\EntityFieldManager $entity_field_manager */
    $entity_field_manager = \Drupal::service('entity_field.manager');

    $ref_options = [];
    $vocabularies = \Drupal::service('entity_type.bundle.info')->getAllBundleInfo()['taxonomy_term'];
    foreach ($vocabularies as $bundle => $vocab_info) {
      $fields = $entity_field_manager->getFieldDefinitions('taxonomy_term', $bundle);
      foreach ($fields as $field_name => $field) {
        // We reject any fields that are \Drupal\Core\Field\BaseFieldDefinition
        // in the first check so we know that the field variable is FieldConfig
        /** @var \Drupal\field\Entity\FieldConfig $field */
        if (
          $field instanceof FieldConfig &&
          $field->getType() === "entity_reference" &&
          ($field->getItemDefinition()->getSetting('target_type') ?? '' === 'taxonomy_term')
          // Note: These are lies.
          // $field->getTargetEntityTypeId() // claims taxonomy term when a user etc.
          // $field->getTargetBundle() // claims null 
        ) {
          $targets = $field->getItemDefinition()->getSetting('handler_settings')['target_bundles'] ?? [];
          $value = "$bundle.$field_name";
          // No toString() method but concat must cause a render() on getLabel.
          foreach ($targets as $target) {
            $value .= ".$target";
            $label = $vocab_info["label"] . " » " . $field->getLabel() . " (" . $target . ")";
            $ref_options[$value] = $label;
          }
        }
      }
    }

    /** @var \Drupal\Core\Entity\EntityTypeManager $entity_type_manager */
    $entity_type_manager = \Drupal::service('entity_type.manager');

    $form['vocabulary_reference'] = [
      '#type' => 'select',
      '#title' => $this->t('Display vocabulary and filter vocabulary field'),
      '#description' => $this->t('The display vocabulary and the field on the display vocabulary that references the filtering vocabulary.'),
      '#options' => $ref_options,
      '#default_value' => $config->get('vocabulary_reference'),
      '#empty_value' => '',
    ];  

    $prepare_entity_options = [];
    $entity_types = \Drupal::service('entity_type.bundle.info')->getAllBundleInfo();
    foreach ($entity_types as $entity_type => $bundles) {
      if ($entity_type_manager->getDefinition($entity_type) instanceof ContentEntityType) {
        foreach ($bundles as $bundle => $bundle_info) {
          $fields = $entity_field_manager->getFieldDefinitions($entity_type, $bundle);
          foreach ($fields as $field_name => $field) {
            // We reject any fields that are \Drupal\Core\Field\BaseFieldDefinition
            // in the first check so we know that the field variable is FieldConfig
            /** @var \Drupal\field\Entity\FieldConfig $field */
            if (
              $field instanceof FieldConfig &&
              $field->getType() === "entity_reference" &&
              ($field->getItemDefinition()->getSetting('target_type') ?? '' === 'taxonomy_term')
              // Note: These are lies.
              // $field->getTargetEntityTypeId() // claims taxonomy term when a user etc.
              // $field->getTargetBundle() // claims null
            ) {
              $targets = $field->getItemDefinition()->getSetting('handler_settings')['target_bundles'] ?? [];
              $value = "$entity_type.$bundle.$field_name";
              // No toString() method but concat must cause a render() on getLabel.
              foreach ($targets as $target) {
                $value .= ".$target";
                $label = $entity_type . ": " . $bundle_info["label"] . " » " . $field->getLabel() . " (" . $target . ")";
                // We group by bundle on the first pass so we can remove singles.
                $prepare_entity_options["$entity_type.$bundle"][$value] = $label;
              }
            }
          }
        }
      }
    }
    $entity_options = [];
    foreach ($prepare_entity_options as $set => $values) {
      if (count($values) > 1) {
        foreach ($values as $value => $label) {
          $entity_options[$value] = $label;
        }
      }
    }

    $form['entity_source_field'] = [
      '#type' => 'select',
      '#title' => $this->t('Source entity bundle and field'),
      '#description' => $this->t('The display vocabulary field that should be used to look up the filter vocabulary to automatically save when this entity is saved.'),
      '#options' => $entity_options,
      '#default_value' => $config->get('entity_source_field'),
      // Needed for this field to really be optional at all (easiest way to disable module without uninstalling it).
      '#empty_value' => '',
    ];
    $form['entity_target_field'] = [
      '#type' => 'select',
      '#title' => $this->t('Target entity bundle and field'),
      '#description' => $this->t('The filter vocabulary field that should receive values from the display when this entity is saved.  This field should be configured <em>not</em> to be shown on the content form and very likely not on content display either; the expectation is the source field will be used with the "Full hierarchy display with link from other vocabulary" field formatter which draws on the "Display vocabulary and filter vocabulary field" setting above (although using the formatter is optional with the autosave capability here and vice versa).  This target field must be on the same entity and bundle (e.g. node: Article) as the "Source entity bundle and field" above. @TODO enforce this.'),
      '#options' => $entity_options,
      '#default_value' => $config->get('entity_target_field'),
      '#empty_value' => '',
    ];

    return parent::buildForm($form, $form_state);
  }

  /** 
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // Retrieve the configuration.
    $this->config(static::SETTINGS)
      // Set the submitted configuration setting.
      ->set('vocabulary_reference', $form_state->getValue('vocabulary_reference'))
      ->set('entity_source_field', $form_state->getValue('entity_source_field'))
      ->set('entity_target_field', $form_state->getValue('entity_target_field'))
      ->save();

    parent::submitForm($form, $form_state);
  }

}