<?php

namespace Drupal\dofacshs\Plugin\Field\FieldFormatter;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\cshs\Plugin\Field\FieldFormatter\CshsFullHierarchyFormatter;
use Drupal\taxonomy\Entity\Term;
use Drupal\taxonomy\TermInterface;

/**
 * Defines the field formatter.
 *
 * @FieldFormatter(
 *   id = "dofacshs_full_hierarchy",
 *   label = @Translation("Full hierarchy display with link from other vocabulary"),
 *   description = @Translation("Display the full hierarchy of the taxonomy term while linking the displayed value to the entity reference filtering links of another separate non-hierarchical vocabulary that the hierarchical vocabulary is connected to by reference fields on its terms."),
 *   field_types = {
 *     "entity_reference",
 *   },
 * )
 */
class DofaCshsFullHierarchyFormatter extends CshsFullHierarchyFormatter {

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode): array {
    // We modify the code from CshsFullHierarchyFormatter rather than use
    // parent::viewElements() because we need to set the terms to NOT be linked
    // so we can wrap it all in our other-vocabulary link and also allow
    // alternative labels for parent terms.
    $elements = [];
    $linked = $this->getSetting('linked');
    $reverse = $this->getSetting('reverse');
    $separator = $this->getSetting('separator') ?: ' ';

    foreach ($this->getEntitiesToView($items, $langcode) as $delta => $term) {
      // Really only fighting with a broken IDE here, not working anyway.
      if ($term instanceof TermInterface) {
        /** \Drupal\taxonomy\TermInterface $term */
        $text = \implode($separator, $this->getTermsAltLabels($this->getTermParents($term, !$reverse), $reverse));
      }
    }

    if ($linked && $url = $this->getMostSpecificUrl($term, $reverse)) {
      $url_string = $url->toString();
      $elements[$delta]['#markup'] = '<a href="' . $url_string . '">' . $text . '</a>';
    }
    else {
      $elements[$delta]['#markup'] = $text;
    }
    return $elements;
  }

  /**
   * Returns the list of terms labels or the alternate name for parents if set.
   *
   * Our replacement of CshsFormatterBase's getTermsLabels().
   *
   * @param \Drupal\taxonomy\TermInterface[] $list
   *   The list of terms.
   * @param bool $reverse
   *   Whether the lowest level of the hierarchy will be first (TRUE) or last.
   *
   * @return string[]|\Drupal\Core\StringTranslation\TranslatableMarkup[]
   *   The list of labels.
   */
  public function getTermsAltLabels(array $list, bool $reverse): array {
      $terms = [];
    
    // @TODO allow setting in config and get from config.
    $altname_field = 'field_alternate_name_if_parent';

    $key_no_alt = $reverse ? 1 : count($list);
    $key = 1;
    foreach ($list as $item) {
      $item = $this->getTranslationFromContext($item);
      if ($key === $key_no_alt) {
        $label = $item->label();
      }
      else {
        $label = $item->$altname_field->value ?? $item->label();
        if ($label === '<hide>') {
          $label = '';
        }
      }
      if ($label) {
        $terms[] = $label;
      }
      $key++;
    }

    return $terms;
  }

  public function getMostSpecificUrl(TermInterface $term, bool $reverse) {
    $vocabulary_reference = \Drupal::config('dofacshs.settings')->get('vocabulary_reference');
    if ($vocabulary_reference) {
      [$display_vocab, $filter_field, $filter_vocab] = explode('.', $vocabulary_reference);
    }
    if ($term->vid->target_id !== $display_vocab) {
      // We have a configuration mismatch and should bail.
      // @TODO give a warning about this or directly restrict configuring the
      // display formatter if this is not valid.
      return NULL;
    }
    // Ex: $filter_field = 'field_filter_location';
    $items = $this->getTermParents($term, !$reverse);
    foreach ($items as $item) {
      /** @var \Drupal\taxonomy\TermInterface $item */
      // Maybe we should be doing hasField() instead of the shortcut?
      $filter_term_id = $item->$filter_field->target_id ?? NULL;
      if ($filter_term_id && $filter_term = Term::load($filter_term_id)) {
        // Ideally here we would be loading each term using
        // EntityReferenceFilteredLinkFormatter with that module extended to
        // have an option to return only the URL.  But for now we simply steal
        // back our facet_pretty_paths contribution that has not been brought
        // in yet anyway.  Er we need to allow setting the view URL here.  Oof
        // we should be extending that formatter as it is the more complex one.
        // @TODO put these in our configuration
        // $view_url = $this->getSetting('view_url');
        // $argument = $this->getSetting('argument_name');
        $view_url = '/projects';
        $argument = $filter_vocab;
        $pathauto_label = \Drupal::service('pathauto.alias_cleaner')->cleanString($filter_term->label());
        $facet_path = $view_url . '/' . $argument . '/' . $pathauto_label;
        $url = \Drupal\Core\Url::fromUri('internal:' . $facet_path);
        // We return with the first one we find rather than complete the loop.
        return $url;
      }
    }
  }
}